-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.27					---
--- Version: 1.2 [2014.10.25]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/guildleveltooltip
--- WoWInterface	http://www.wowinterface.com/downloads/info20527-GuildPointsTooltip.html
--- Request			http://www.curse.com/forums/games/world-of-warcraft/addons/148517-got-a-request

-- Thanks to Cowmonster/Isoloedlk (US-Khaz Modan) for a great amount of help with the inspect code

local NAME = ...
local VERSION = GetAddOnMetadata(NAME, "Version")

local db
local format = format

local playerRealm = GetRealmName()
local f = CreateFrame("Frame")

	--------------
	--- Events ---
	--------------

function f:ADDON_LOADED(name)
	if name ~= NAME then return end
	
	GuildPointsTooltipDB = setmetatable(GuildPointsTooltipDB or {}, {
		__index = function(t, k)
			local v = {}
			rawset(t, k, v)
			return v
		end
	})
	GuildPointsTooltipDB.version = VERSION
	self:UnregisterEvent("ADDON_LOADED")
end

local newUnit = "target"

local function Notify(unit)
	if CanInspect(unit) and time() > (f.cd or 0) then
		-- NotifyInspect() is throttled by 6 calls for every 10 sec
		f.cd = time() + 1; newUnit = unit
		
		if not InspectFrame or (InspectFrame and not InspectFrame:IsShown()) then
			NotifyInspect(unit)
		end
	end
end

function f:UPDATE_MOUSEOVER_UNIT()
	Notify("mouseover")
end

function f:PLAYER_TARGET_CHANGED()
	Notify("target")
end

function f:INSPECT_READY(guid)
	-- Blizzard_InspectUI is loaded and shown
	if InspectFrame and InspectFrame:IsShown() then
		newUnit = "target" -- unit is mostly "target" on manual inspect
	end
	
	-- INSPECT_READY passes the unit's guid instead of the unit id
	if UnitGUID(newUnit) == guid then
		-- guild level removed in WoD (6.0.2)
		local chievos, members, guildrealm = GetInspectGuildInfo(newUnit)
		if members == 0 then return end
		
		local guild, realm = strmatch(guildrealm, "(.+)%-(.+)")
		if not guild then guild, realm = guildrealm, playerRealm end
		if not guild then return end
		
		db = GuildPointsTooltipDB[realm]
		local pendingUpdate = not db[guild]
		
		db[guild] = db[guild] or {}
		db[guild][1] = chievos
		db[guild][2] = members
		db[guild][3] = time()
		db[guild][4] = date("%Y.%m.%d %H:%M")
		
		if pendingUpdate then
			f.OnTooltipSetUnit(GameTooltip)
		end
	end
end

function f:OnEvent(event, ...)
	self[event](self, ...)
end

local events = {
	"ADDON_LOADED",
	"UPDATE_MOUSEOVER_UNIT",
	"PLAYER_TARGET_CHANGED",
	"INSPECT_READY"
}

for _, v in ipairs(events) do
	f:RegisterEvent(v)
end

f:SetScript("OnEvent", f.OnEvent)

	------------
	--- Hook ---
	------------

-- GameTooltip gets passed, self refers to that instead of f
function f:OnTooltipSetUnit()
	local _, unit = self:GetUnit()
	if not unit then return end
	
	local guild, _, _, realm = GetGuildInfo(unit)
	db = GuildPointsTooltipDB[realm or playerRealm]
	if not db[guild] then return end
	
	self:AddLine(db[guild][1].." "..ACHIEVEMENT_POINTS)
	self:AddLine(format(INSPECT_GUILD_NUM_MEMBERS, db[guild][2]))
	self:Show() -- update tooltip
end

GameTooltip:HookScript("OnTooltipSetUnit", f.OnTooltipSetUnit)
